@@ -30,16 +30,19 @@ import io
 
 from pynlpl.statistics import FrequencyList
 from collections import defaultdict
-import numpy as np
+try:
+    import numpy as np
+except ImportError:
+    np = None
 import subprocess
 import itertools
 import time
 import random
+import math
 import copy
 import datetime
 import os.path
 
-
 def auc(x, y, reorder=False): #from sklearn, http://scikit-learn.org, licensed under BSD License
     """Compute Area Under the Curve (AUC) using the trapezoidal rule
 
@@ -77,6 +80,9 @@ def auc(x, y, reorder=False): #from sklearn, http://scikit-learn.org, licensed u
     auc_score : Computes the area under the ROC curve
 
     """
+    if np is None:
+        raise ImportError("No numpy installed")
+
     # XXX: Consider using  ``scipy.integrate`` instead, or moving to
     # ``utils.extmath``
     if not isinstance(x, np.ndarray): x = np.array(x)
@@ -101,6 +107,19 @@ def auc(x, y, reorder=False): #from sklearn, http://scikit-learn.org, licensed u
     return area
 
 
+def mae(absolute_error_values):
+    if np is None:
+        return sum(absolute_error_values) / len(absolute_error_values)
+    else:
+        return np.mean(absolute_error_values)
+
+def rmse(squared_error_values):
+    if np is None:
+        return math.sqrt(sum(squared_error_values)/len(squared_error_values))
+    else:
+        return math.sqrt(np.mean(squared_error_values))
+
+
 class ProcessFailed(Exception):
     pass
 
@@ -375,6 +394,34 @@ class ClassEvaluation(object):
         return str(self)
 
 
+class OrdinalEvaluation(ClassEvaluation):
+
+    def __init__(self,  goals = [], observations = [], missing = {}, encoding ='utf-8'):
+        ClassEvaluation.__init__(self,goals,observations,missing,encoding='utf-8')
+
+    def compute(self):
+        assert not False in [type(cls) == int for cls in self.classes]
+        ClassEvaluation.compute(self)
+        self.error = defaultdict(list)
+        self.squared_error = defaultdict(list)
+        for goal, observation in self:
+            self.error[observation].append(abs(goal-observation))
+            self.squared_error[observation].append(abs(goal-observation)**2)
+
+    def mae(self, cls=None):
+        if not self.computed: self.compute()
+        if cls:
+            return mae(self.error[cls])
+        else:
+            return mae(sum([self.error[x] for x in set(self.goals)], []))
+             
+    def rmse(self, cls=None):
+        if not self.computed: self.compute()
+        if cls:
+            return rmse(self.squared_error[cls])
+        else:
+            return rmse(sum([self.squared_error[x] for x in set(self.goals)], []))
+
 class AbstractExperiment(object):
 
     def __init__(self, inputdata = None, **parameters):
@@ -661,18 +708,18 @@ def filesampler(files, testsetsize = 0.1, devsetsize = 0, trainsetsize = 0, outp
         devset = {}
         for i in range(1,total+1):
             trainset[i] = True
-        for i in random.sample(trainset.keys(), testsetsize):
+        for i in random.sample(trainset.keys(), int(testsetsize)):
             testset[i] = True
             del trainset[i]
 
         if devsetsize > 0:
-            for i in random.sample(trainset.keys(), devsetsize):
+            for i in random.sample(trainset.keys(), int(devsetsize)):
                 devset[i] = True
                 del trainset[i]
 
         if trainsetsize > 0:
             newtrainset = {}
-            for i in random.sample(trainset.keys(), trainsetsize):
+            for i in random.sample(trainset.keys(), int(trainsetsize)):
                 newtrainset[i] = True
             trainset = newtrainset
 
